/*
bthash.h Version 0.1.0  Bthash header file
Copyright (C) 2003  dondalah@ripco.com (Dondalah)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to:

	Free Software Foundation, Inc.
	59 Temple Place - Suite 330
	Boston, MA  02111-1307, USA.
*/

/* Linux version */

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>

#include <sys/file.h>

#include <fcntl.h>
#include <errno.h>
#include <unistd.h>
#include <string.h>

#define BT_SIGNATURE "xOoCKftE5hjUXeGB"
#define H_SIGNATURE  "BUHJnOdAh6VgimtY"

#define ROOT   256
#define HDRLEN 256
#define UINT_MAX  0xffffffff
#define BTREE_ERR 300000000
#define HASH_ERR  350000000
#define SYS_ERR   400000000
#define RND_BYTE 15
#define DELETED 255
/* open read only or read/write */
#define READ 0
#define UPDT 1
/* open with lock */
#define NOLOCK 0
#define LOCK 1
#define TSTLOCK 2
/* btlock */
#define LOCKGOOD 0
/* lock status changed to LOCK */
#define BTLOCK1  1
/* database locked by another user */
#define BTLOCK3  3
/* lock error */
#define BTLOCK4  4
/* btunlock */
#define UNLOCKGOOD 0
/* lock status changed to NOLOCK */
#define BTUNLOCK1  1
/* unlock error */
#define BTUNLOCK4  4
/* btisrt */
#define ISRTGOOD 0
#define ISRTDUP  1
#define ISRTLOCK3 3
#define ISRTLOCK4 4
/* btupdt */
#define UPDTGOOD 0
#define UPDTERR  1
#define UPDTLOCK3 3
#define UPDTLOCK4 4
/* btdlet */
#define DLETGOOD 0
#define DLETERR  1
#define DLETLOCK3 3
#define DLETLOCK4 4
/* btdbck */
#define DBCKGOOD 0
/* btget */
/* btgetnext */
/* btgetprev */
#define FOUND 0
#define NOTFOUND 1
#define BTEOF 2
#define BTBOF 3
#define LEAFNODE 4
#define INDXNODE 5
#define MATCHED 6
#define FRSTPAPA 7
/* btsweep */
#define SWPNEXT 0
#define SWPFRST 1
#define SWPFOUND 0
#define SWPEOF   1
#define SWPERR   2
/* btinit options */
/* truncate old file & initialize */
#define INITOLD 1
/* create new file & initialize */
#define INITNEW 2
/* truncate old file & initialize */
/* or create new file & initialize */
#define INITALL 3

typedef struct bt_nodefmt {
   off_t left;
   } bt_nodefmt;

typedef struct h_nodefmt {
   off_t next;
   } h_nodefmt;

typedef struct blkfmt {
   off_t right;
   int count;
   } blkfmt;

typedef struct stkfmt {
   struct stkfmt *prev;
   struct stkfmt *next;
   off_t ofst;
   } stkfmt;

typedef struct btreefmt {
   char signature[16];
   char sig_end[4];
   int blksz;
   int nodesz;   /* records per node */
   int rcdlen;
   int keylen;
   int keyofst;
   int dletofst;
   int blkhdr;
   int rcdhdr;
   int rootofst;
   off_t nxtfreeblk;
   int hndl;
   int rcdnode;
   int blknode;
   int midofst;
   int seq;
   int lock;
   int rwflag;
   int reason;
   int rcdnum;          /* record number in block */
   unsigned char *rcd;  /* record address */
   unsigned char *key;  /* key    address */
   unsigned char *dletbyte;  /* delete byte address */
   unsigned char *rndSeed;
   off_t ofst;
   off_t ofstleft;
   off_t ofstrght;
   stkfmt *head;
   stkfmt *tail;
   bt_nodefmt *node;
   blkfmt *blk;
   blkfmt *blkleft;
   blkfmt *blkrght;
   char dbname[128];
   } btreefmt;

typedef struct hashfmt {
   char signature[16];
   char sig_end[4];
   int totrcds;
   int rcdlen;
   int keylen;
   int keyofst;
   int rcdhdr;
   int hdrlen;
   off_t freestk;
   int hndl;
   unsigned char *rcd;
   unsigned char *key;
   int rcdnode;
   int lock;
   int reason;
   int seq;
   int fsofst;  /* offset of freestk in header */
   off_t hashofst;
   off_t hashend;
   off_t dbend;
   off_t ofst;
   off_t ofstnext;
   off_t ofstprev;
   h_nodefmt *node;
   h_nodefmt *nodenext;
   h_nodefmt *nodeprev;
   char dbname[128];
   } hashfmt;

/***********************************************************************/
/*                      Btree Subroutines                              */
/***********************************************************************/

void btclose(btreefmt *bthdr);

void btcloset(btreefmt *bthdr);

int  btdbck(btreefmt *bthdr);

void btdbsz(btreefmt *bthdr, int *blocks,
	int *count);

int  btdepth(btreefmt *bthdr);

int  btdlet(btreefmt *bthdr, off_t ofst, unsigned char *key);

void btdlets(btreefmt *bthdr, int *blocks,
	int *count, int *leaf, int *nonleaf,
	int *active, int *deletes,
	int *indxrcd, int *leafrcd);

void btflustk(btreefmt *bthdr);

int  btget(btreefmt *bthdr, off_t ofst,
	unsigned char *key, unsigned char *buf);

void btgetblk(btreefmt *bthdr, off_t ofst);

int  btgetnxt(btreefmt *bthdr, off_t ofst,
	unsigned char *key, unsigned char *buf);

int  btgetp(btreefmt *bthdr, off_t ofst,
	unsigned char *key, unsigned char *buf);

int  btgetprv(btreefmt *bthdr, off_t ofst,
	unsigned char *key, unsigned char *buf);

void bthxdmp(btreefmt *bthdr, char *tag);

void bthxmem(int len, unsigned char *memory);

int btinit(char *dbname, int truncflag, int nodesz,
	int rcdlen, int keylen,
	int keyofst, int dletofst);

int btisrt(btreefmt *bthdr, off_t ofst, unsigned char *rcd);

int btlock(btreefmt *bthdr);

off_t btnewblk(btreefmt *bthdr);

btreefmt *btopen(char *dbname, int rwflag,
   int lock, int *reason);

btreefmt *btopent(int nodesz, int rcdlen,
   int keylen, int keyofst, int dletofst,
   int *reason, unsigned char *seed);

void btoptim(int totrcds, int nodesz,
	int *levels, int *lvltot,
	int *lvlrem, int *lvlblks,
	int *blktot);

void btpop(btreefmt *bthdr);

void btpush(btreefmt *bthdr, off_t ofst);

void btputblk(btreefmt *bthdr, off_t ofst, blkfmt *blk);

void btputhx(int adr, int len, unsigned char *str);

int btreorg(btreefmt *bthdr);

void btrevers(btreefmt *bthdr, off_t ofst);

off_t btseek(btreefmt *bthdr, off_t ofst);

void btshwstk(btreefmt *bthdr);

void btsplit(btreefmt *bthdr);

void btspltrt(btreefmt *bthdr);

int  btstkdp(btreefmt *bthdr);

stkfmt *btstkini(btreefmt *bthdr);

int btsweep(btreefmt *bthdr, int position,
   unsigned char *buf);

void  bttravgt(btreefmt *bthdr, off_t ofst, blkfmt *blk);

void  bttravrs(btreefmt *bthdr, off_t ofst);

off_t bttravsk(btreefmt *bthdr, off_t ofst);

int btunlock(btreefmt *bthdr);

int btupdt(btreefmt *bthdr, off_t ofst, unsigned char *rcd);

/***********************************************************************/
/*                      Hash Subroutines                               */
/***********************************************************************/

void getmd5(unsigned char *key,
   int keylen, unsigned char *digest);

off_t hash(hashfmt *hshhdr, unsigned char *key);

int  hchain(hashfmt *hshhdr, int *count,
   int *smlen, int *smlensq, int *freelen);

int  hclose(hashfmt *hshhdr);

void hcloset(hashfmt *hshhdr);

int hdbck(hashfmt *hshhdr);

int hdlet(hashfmt *hshhdr, unsigned char *key);

int hget(hashfmt *hshhdr, unsigned char *key,
   unsigned char *buf);

off_t hgetptr(hashfmt *hshhdr, off_t ofst);

void hgetblk(hashfmt *hshhdr, off_t ofst);

void hhxdmp(hashfmt *hshhdr, char *tag);

void hhxmem(int len, unsigned char *memory);

int hinit(char *dbname, int truncflag, int nodesz,
	int rcdlen, int keylen, int keyofst);

int hisrt(hashfmt *hshhdr, unsigned char *rcd);

int hlock(hashfmt *hshhdr);

off_t hnewblk(hashfmt *hshhdr);

hashfmt *hopen(char *dbname, int rwflag,
   int lock, int *reason);

hashfmt *hopent(int nodesz, int rcdlen,
   int keylen, int keyofst,
   int *reason, unsigned char *seed);

int  hprime(int est_prime);

void hputblk(hashfmt *hshhdr, off_t ofst, h_nodefmt *blk);

void hputptr(hashfmt *hshhdr, off_t ofst, off_t blkofst);

void hputhx(int adr, int len, unsigned char *str);

int hreorg(hashfmt *hshhdr, int totrcds,
   double freespc);

off_t hseek(hashfmt *hshhdr, off_t ofst);

void hsize(hashfmt *hshhdr, int *rcds,
   int *ovfl, int *empty, int *freeblk);

int hsweep(hashfmt *hshhdr, int position,
   unsigned char *buf);

int hunlock(hashfmt *hshhdr);

int hupdt(hashfmt *hshhdr, unsigned char *rcd);

/* static unsigned char *rndSeed; */

int rnd(unsigned char *rndSeed);

unsigned char *rndinit(void);

unsigned int rndnum(unsigned int lmt, unsigned char *rndSeed);

unsigned int rndpwr(int bits, unsigned char *rndSeed);
